#!/usr/bin/python3
# -*- coding: utf-8 -*-

class ExchangePyError(Exception):
    pass


class UnknownEventType(ExchangePyError):
    message = "ExchangePy doesn't handle this event type"


class ExchangeError(ExchangePyError):
    pass


class QueryCanceled(ExchangePyError):
    pass


class HTTPError(ExchangePyError):
    code = 400
    message = "Malformed request"


class WAFLimitViolated(HTTPError):
    code = 403
    message = "The WAF Limit (Web Application Firewall) has been violated."


class RateLimitReached(HTTPError):
    code = 429
    message = "The rate limit has been reached."


class IPAddressBanned(HTTPError):
    code = 418
    message = "Your IP address has been auto-banned for continuing to send requests after receiving 429 codes."



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: exchanges_wrapper/api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x65xchanges_wrapper/api.proto\x12\x06martin\"\x83\x01\n\x19\x46\x65tchFundingWalletRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\r\n\x05\x61sset\x18\x03 \x01(\t\x12\x1a\n\x12need_btc_valuation\x18\x04 \x01(\x08\x12\x16\n\x0ereceive_window\x18\x05 \x01(\x03\"\xd0\x01\n\x1a\x46\x65tchFundingWalletResponse\x12=\n\x08\x62\x61lances\x18\x01 \x03(\x0b\x32+.martin.FetchFundingWalletResponse.Balances\x1as\n\x08\x42\x61lances\x12\r\n\x05\x61sset\x18\x01 \x01(\t\x12\x0c\n\x04\x66ree\x18\x02 \x01(\t\x12\x0e\n\x06locked\x18\x03 \x01(\t\x12\x0e\n\x06\x66reeze\x18\x04 \x01(\t\x12\x13\n\x0bwithdrawing\x18\x05 \x01(\t\x12\x15\n\rbtc_valuation\x18\x06 \x01(\t\"\xa6\x02\n\x13\x43\x61ncelOrderResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x19\n\x11origClientOrderId\x18\x02 \x01(\t\x12\x0f\n\x07orderId\x18\x03 \x01(\x04\x12\x13\n\x0borderListId\x18\x04 \x01(\x05\x12\x15\n\rclientOrderId\x18\x05 \x01(\t\x12\x14\n\x0ctransactTime\x18\x06 \x01(\x04\x12\r\n\x05price\x18\x07 \x01(\t\x12\x0f\n\x07origQty\x18\x08 \x01(\t\x12\x13\n\x0b\x65xecutedQty\x18\t \x01(\t\x12\x1b\n\x13\x63ummulativeQuoteQty\x18\n \x01(\t\x12\x0e\n\x06status\x18\x0b \x01(\t\x12\x13\n\x0btimeInForce\x18\x0c \x01(\t\x12\x0c\n\x04type\x18\r \x01(\t\x12\x0c\n\x04side\x18\x0e \x01(\t\"[\n\x12\x43\x61ncelOrderRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x10\n\x08order_id\x18\x04 \x01(\x03\"\x90\x02\n\x18\x43reateLimitOrderResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0f\n\x07orderId\x18\x02 \x01(\x04\x12\x13\n\x0borderListId\x18\x03 \x01(\x11\x12\x15\n\rclientOrderId\x18\x04 \x01(\t\x12\x14\n\x0ctransactTime\x18\x05 \x01(\x04\x12\r\n\x05price\x18\x06 \x01(\t\x12\x0f\n\x07origQty\x18\x07 \x01(\t\x12\x13\n\x0b\x65xecutedQty\x18\x08 \x01(\t\x12\x1b\n\x13\x63ummulativeQuoteQty\x18\t \x01(\t\x12\x0e\n\x06status\x18\n \x01(\t\x12\x13\n\x0btimeInForce\x18\x0b \x01(\t\x12\x0c\n\x04type\x18\x0c \x01(\t\x12\x0c\n\x04side\x18\r \x01(\t\"\x9e\x01\n\x17\x43reateLimitOrderRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x10\n\x08\x62uy_side\x18\x04 \x01(\x08\x12\x10\n\x08quantity\x18\x05 \x01(\t\x12\r\n\x05price\x18\x06 \x01(\t\x12\x1b\n\x13new_client_order_id\x18\x07 \x01(\x03\"\xf8\x05\n\x15OnOrderUpdateResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x17\n\x0f\x63lient_order_id\x18\x02 \x01(\t\x12\x0c\n\x04side\x18\x03 \x01(\t\x12\x12\n\norder_type\x18\x04 \x01(\t\x12\x15\n\rtime_in_force\x18\x05 \x01(\t\x12\x16\n\x0eorder_quantity\x18\x06 \x01(\t\x12\x13\n\x0border_price\x18\x07 \x01(\t\x12\x12\n\nstop_price\x18\x08 \x01(\t\x12\x18\n\x10iceberg_quantity\x18\t \x01(\t\x12\x15\n\rorder_list_id\x18\n \x01(\x11\x12\x1a\n\x12original_client_id\x18\x0b \x01(\t\x12\x16\n\x0e\x65xecution_type\x18\x0c \x01(\t\x12\x14\n\x0corder_status\x18\r \x01(\t\x12\x1b\n\x13order_reject_reason\x18\x0e \x01(\t\x12\x10\n\x08order_id\x18\x0f \x01(\x04\x12\x1e\n\x16last_executed_quantity\x18\x10 \x01(\t\x12\"\n\x1a\x63umulative_filled_quantity\x18\x11 \x01(\t\x12\x1b\n\x13last_executed_price\x18\x12 \x01(\t\x12\x19\n\x11\x63ommission_amount\x18\x13 \x01(\t\x12\x18\n\x10\x63ommission_asset\x18\x14 \x01(\t\x12\x18\n\x10transaction_time\x18\x15 \x01(\x04\x12\x10\n\x08trade_id\x18\x16 \x01(\x12\x12\x10\n\x08ignore_a\x18\x17 \x01(\x04\x12\x15\n\rin_order_book\x18\x18 \x01(\x08\x12\x15\n\ris_maker_side\x18\x19 \x01(\x08\x12\x10\n\x08ignore_b\x18\x1a \x01(\x08\x12\x1b\n\x13order_creation_time\x18\x1b \x01(\x04\x12\x1e\n\x16quote_asset_transacted\x18\x1c \x01(\t\x12#\n\x1blast_quote_asset_transacted\x18\x1d \x01(\t\x12\x1c\n\x14quote_order_quantity\x18\x1e \x01(\t\"&\n\x15OnFundsUpdateResponse\x12\r\n\x05\x66unds\x18\x01 \x01(\t\"t\n\x14OnFundsUpdateRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x12\n\nbase_asset\x18\x04 \x01(\t\x12\x13\n\x0bquote_asset\x18\x05 \x01(\t\"!\n\x0eSimpleResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"e\n\x16OnTickerUpdateResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x12\n\nopen_price\x18\x02 \x01(\t\x12\x13\n\x0b\x63lose_price\x18\x03 \x01(\t\x12\x12\n\nevent_time\x18\x04 \x01(\x04\"\xbd\x02\n\x18\x41\x63\x63ountTradeListResponse\x12\x35\n\x05items\x18\x01 \x03(\x0b\x32&.martin.AccountTradeListResponse.Trade\x1a\xe9\x01\n\x05Trade\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x04\x12\x0f\n\x07orderId\x18\x03 \x01(\x04\x12\x13\n\x0borderListId\x18\x04 \x01(\x11\x12\r\n\x05price\x18\x05 \x01(\t\x12\x0b\n\x03qty\x18\x06 \x01(\t\x12\x10\n\x08quoteQty\x18\x07 \x01(\t\x12\x12\n\ncommission\x18\x08 \x01(\t\x12\x17\n\x0f\x63ommissionAsset\x18\t \x01(\t\x12\x0c\n\x04time\x18\n \x01(\x04\x12\x0f\n\x07isBuyer\x18\x0b \x01(\x08\x12\x0f\n\x07isMaker\x18\x0c \x01(\x08\x12\x13\n\x0bisBestMatch\x18\r \x01(\x08\"q\n\x17\x41\x63\x63ountTradeListRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\r\n\x05limit\x18\x04 \x01(\r\x12\x12\n\nstart_time\x18\x05 \x01(\x03\"%\n\x13\x46\x65tchKlinesResponse\x12\x0e\n\x06klines\x18\x01 \x03(\t\"J\n\x16OnKlinesUpdateResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08interval\x18\x02 \x01(\t\x12\x0e\n\x06\x63\x61ndle\x18\x03 \x01(\t\"j\n\x12\x46\x65tchKlinesRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x10\n\x08interval\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\r\"\xb7\x03\n(FetchTickerPriceChangeStatisticsResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x13\n\x0bpriceChange\x18\x02 \x01(\t\x12\x1a\n\x12priceChangePercent\x18\x03 \x01(\t\x12\x18\n\x10weightedAvgPrice\x18\x04 \x01(\t\x12\x16\n\x0eprevClosePrice\x18\x05 \x01(\t\x12\x11\n\tlastPrice\x18\x06 \x01(\t\x12\x0f\n\x07lastQty\x18\x07 \x01(\t\x12\x10\n\x08\x62idPrice\x18\x08 \x01(\t\x12\x0e\n\x06\x62idQty\x18\t \x01(\t\x12\x10\n\x08\x61skPrice\x18\n \x01(\t\x12\x0e\n\x06\x61skQty\x18\x0b \x01(\t\x12\x11\n\topenPrice\x18\x0c \x01(\t\x12\x11\n\thighPrice\x18\r \x01(\t\x12\x10\n\x08lowPrice\x18\x0e \x01(\t\x12\x0e\n\x06volume\x18\x0f \x01(\t\x12\x13\n\x0bquoteVolume\x18\x10 \x01(\t\x12\x10\n\x08openTime\x18\x11 \x01(\x04\x12\x11\n\tcloseTime\x18\x12 \x01(\x04\x12\x0f\n\x07\x66irstId\x18\x13 \x01(\x04\x12\x0e\n\x06lastId\x18\x14 \x01(\x04\x12\r\n\x05\x63ount\x18\x15 \x01(\x04\"?\n\x1e\x46\x65tchSymbolPriceTickerResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t\"J\n\x16\x46\x65tchOrderBookResponse\x12\x14\n\x0clastUpdateId\x18\x01 \x01(\x04\x12\x0c\n\x04\x62ids\x18\x02 \x03(\t\x12\x0c\n\x04\x61sks\x18\x03 \x03(\t\"\x96\x01\n\x1b\x46\x65tchAccountBalanceResponse\x12>\n\x08\x62\x61lances\x18\x01 \x03(\x0b\x32,.martin.FetchAccountBalanceResponse.Balances\x1a\x37\n\x08\x42\x61lances\x12\r\n\x05\x61sset\x18\x01 \x01(\t\x12\x0c\n\x04\x66ree\x18\x02 \x01(\t\x12\x0e\n\x06locked\x18\x03 \x01(\t\"\x87\x13\n\x1f\x46\x65tchExchangeInfoSymbolResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x11\n\tbaseAsset\x18\x03 \x01(\t\x12\x1a\n\x12\x62\x61seAssetPrecision\x18\x04 \x01(\r\x12\x12\n\nquoteAsset\x18\x05 \x01(\t\x12\x16\n\x0equotePrecision\x18\x06 \x01(\r\x12\x1b\n\x13quoteAssetPrecision\x18\x07 \x01(\r\x12\x1f\n\x17\x62\x61seCommissionPrecision\x18\x08 \x01(\r\x12 \n\x18quoteCommissionPrecision\x18\t \x01(\r\x12\x12\n\norderTypes\x18\n \x03(\t\x12\x16\n\x0eicebergAllowed\x18\x0b \x01(\x08\x12\x12\n\nocoAllowed\x18\x0c \x01(\x08\x12\"\n\x1aquoteOrderQtyMarketAllowed\x18\r \x01(\x08\x12\x19\n\x11\x61llowTrailingStop\x18\x0e \x01(\x08\x12\x1c\n\x14\x63\x61ncelReplaceAllowed\x18\x0f \x01(\x08\x12\x1c\n\x14isSpotTradingAllowed\x18\x10 \x01(\x08\x12\x1e\n\x16isMarginTradingAllowed\x18\x11 \x01(\x08\x12@\n\x07\x66ilters\x18\x12 \x01(\x0b\x32/.martin.FetchExchangeInfoSymbolResponse.Filters\x12\x13\n\x0bpermissions\x18\x13 \x03(\t\x1a\xd6\x0e\n\x07\x46ilters\x12V\n\x0cprice_filter\x18\x01 \x01(\x0b\x32;.martin.FetchExchangeInfoSymbolResponse.Filters.PriceFilterH\x00\x88\x01\x01\x12X\n\rpercent_price\x18\x02 \x01(\x0b\x32<.martin.FetchExchangeInfoSymbolResponse.Filters.PercentPriceH\x01\x88\x01\x01\x12N\n\x08lot_size\x18\x03 \x01(\x0b\x32\x37.martin.FetchExchangeInfoSymbolResponse.Filters.LotSizeH\x02\x88\x01\x01\x12V\n\x0cmin_notional\x18\x04 \x01(\x0b\x32;.martin.FetchExchangeInfoSymbolResponse.Filters.MinNotionalH\x03\x88\x01\x01\x12X\n\riceberg_parts\x18\x05 \x01(\x0b\x32<.martin.FetchExchangeInfoSymbolResponse.Filters.IcebergPartsH\x04\x88\x01\x01\x12[\n\x0fmarket_lot_size\x18\x06 \x01(\x0b\x32=.martin.FetchExchangeInfoSymbolResponse.Filters.MarketLotSizeH\x05\x88\x01\x01\x12Y\n\x0emax_num_orders\x18\x07 \x01(\x0b\x32<.martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumOrdersH\x06\x88\x01\x01\x12\x62\n\x13max_num_algo_orders\x18\x08 \x01(\x0b\x32@.martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumAlgoOrdersH\x07\x88\x01\x01\x12h\n\x16max_num_iceberg_orders\x18\t \x01(\x0b\x32\x43.martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumIcebergOrdersH\x08\x88\x01\x01\x12V\n\x0cmax_position\x18\n \x01(\x0b\x32;.martin.FetchExchangeInfoSymbolResponse.Filters.MaxPositionH\t\x88\x01\x01\x1aW\n\x0bPriceFilter\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x10\n\x08minPrice\x18\x02 \x01(\t\x12\x10\n\x08maxPrice\x18\x03 \x01(\t\x12\x10\n\x08tickSize\x18\x04 \x01(\t\x1a\x66\n\x0cPercentPrice\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x14\n\x0cmultiplierUp\x18\x02 \x01(\t\x12\x16\n\x0emultiplierDown\x18\x03 \x01(\t\x12\x14\n\x0c\x61vgPriceMins\x18\x04 \x01(\r\x1aO\n\x07LotSize\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x0e\n\x06minQty\x18\x02 \x01(\t\x12\x0e\n\x06maxQty\x18\x03 \x01(\t\x12\x10\n\x08stepSize\x18\x04 \x01(\t\x1a\x63\n\x0bMinNotional\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x13\n\x0bminNotional\x18\x02 \x01(\t\x12\x15\n\rapplyToMarket\x18\x03 \x01(\x08\x12\x14\n\x0c\x61vgPriceMins\x18\x04 \x01(\r\x1a\x31\n\x0cIcebergParts\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\r\x1aU\n\rMarketLotSize\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x0e\n\x06minQty\x18\x02 \x01(\t\x12\x0e\n\x06maxQty\x18\x03 \x01(\t\x12\x10\n\x08stepSize\x18\x04 \x01(\t\x1a\x38\n\x0cMaxNumOrders\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x14\n\x0cmaxNumOrders\x18\x02 \x01(\r\x1a@\n\x10MaxNumAlgoOrders\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x18\n\x10maxNumAlgoOrders\x18\x02 \x01(\r\x1a\x46\n\x13MaxNumIcebergOrders\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x1b\n\x13maxNumIcebergOrders\x18\x02 \x01(\r\x1a\x36\n\x0bMaxPosition\x12\x12\n\nfilterType\x18\x01 \x01(\t\x12\x13\n\x0bmaxPosition\x18\x02 \x01(\tB\x0f\n\r_price_filterB\x10\n\x0e_percent_priceB\x0b\n\t_lot_sizeB\x0f\n\r_min_notionalB\x10\n\x0e_iceberg_partsB\x12\n\x10_market_lot_sizeB\x11\n\x0f_max_num_ordersB\x16\n\x14_max_num_algo_ordersB\x19\n\x17_max_num_iceberg_ordersB\x0f\n\r_max_position\"\xf6\x02\n\x17\x43\x61ncelAllOrdersResponse\x12:\n\x05items\x18\x01 \x03(\x0b\x32+.martin.CancelAllOrdersResponse.CancelOrder\x1a\x9e\x02\n\x0b\x43\x61ncelOrder\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x19\n\x11origClientOrderId\x18\x02 \x01(\t\x12\x0f\n\x07orderId\x18\x03 \x01(\x04\x12\x13\n\x0borderListId\x18\x04 \x01(\x05\x12\x15\n\rclientOrderId\x18\x05 \x01(\t\x12\x14\n\x0ctransactTime\x18\x06 \x01(\x04\x12\r\n\x05price\x18\x07 \x01(\t\x12\x0f\n\x07origQty\x18\x08 \x01(\t\x12\x13\n\x0b\x65xecutedQty\x18\t \x01(\t\x12\x1b\n\x13\x63ummulativeQuoteQty\x18\n \x01(\t\x12\x0e\n\x06status\x18\x0b \x01(\t\x12\x13\n\x0btimeInForce\x18\x0c \x01(\t\x12\x0c\n\x04type\x18\r \x01(\t\x12\x0c\n\x04side\x18\x0e \x01(\t\"v\n\x11\x46\x65tchOrderRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x10\n\x08order_id\x18\x04 \x01(\x03\x12\x1a\n\x12\x66illed_update_call\x18\x05 \x01(\x08\"\xeb\x02\n\x12\x46\x65tchOrderResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0f\n\x07orderId\x18\x02 \x01(\x04\x12\x13\n\x0borderListId\x18\x03 \x01(\x11\x12\x15\n\rclientOrderId\x18\x04 \x01(\t\x12\r\n\x05price\x18\x05 \x01(\t\x12\x0f\n\x07origQty\x18\x06 \x01(\t\x12\x13\n\x0b\x65xecutedQty\x18\x07 \x01(\t\x12\x1b\n\x13\x63ummulativeQuoteQty\x18\x08 \x01(\t\x12\x0e\n\x06status\x18\t \x01(\t\x12\x13\n\x0btimeInForce\x18\n \x01(\t\x12\x0c\n\x04type\x18\x0b \x01(\t\x12\x0c\n\x04side\x18\x0c \x01(\t\x12\x11\n\tstopPrice\x18\r \x01(\t\x12\x12\n\nicebergQty\x18\x0e \x01(\t\x12\x0c\n\x04time\x18\x0f \x01(\x04\x12\x12\n\nupdateTime\x18\x10 \x01(\x04\x12\x11\n\tisWorking\x18\x11 \x01(\x08\x12\x19\n\x11origQuoteOrderQty\x18\x12 \x01(\t\"\xc6\x03\n\x17\x46\x65tchOpenOrdersResponse\x12\x14\n\x0crate_limiter\x18\x01 \x01(\x05\x12\x34\n\x05items\x18\x02 \x03(\x0b\x32%.martin.FetchOpenOrdersResponse.Order\x1a\xde\x02\n\x05Order\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0f\n\x07orderId\x18\x02 \x01(\x04\x12\x13\n\x0borderListId\x18\x03 \x01(\x11\x12\x15\n\rclientOrderId\x18\x04 \x01(\t\x12\r\n\x05price\x18\x05 \x01(\t\x12\x0f\n\x07origQty\x18\x06 \x01(\t\x12\x13\n\x0b\x65xecutedQty\x18\x07 \x01(\t\x12\x1b\n\x13\x63ummulativeQuoteQty\x18\x08 \x01(\t\x12\x0e\n\x06status\x18\t \x01(\t\x12\x13\n\x0btimeInForce\x18\n \x01(\t\x12\x0c\n\x04type\x18\x0b \x01(\t\x12\x0c\n\x04side\x18\x0c \x01(\t\x12\x11\n\tstopPrice\x18\r \x01(\t\x12\x12\n\nicebergQty\x18\x0e \x01(\t\x12\x0c\n\x04time\x18\x0f \x01(\x04\x12\x12\n\nupdateTime\x18\x10 \x01(\x04\x12\x11\n\tisWorking\x18\x11 \x01(\x08\x12\x19\n\x11origQuoteOrderQty\x18\x12 \x01(\t\"D\n\rMarketRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\"\x81\x01\n\x12StartStreamRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x1b\n\x13market_stream_count\x18\x04 \x01(\x05\x12\x19\n\x11user_stream_count\x18\x05 \x01(\x05\"[\n\x1bOpenClientConnectionRequest\x12\x10\n\x08trade_id\x18\x01 \x01(\t\x12\x14\n\x0c\x61\x63\x63ount_name\x18\x02 \x01(\t\x12\x14\n\x0crate_limiter\x18\x03 \x01(\x05\"z\n\x16OpenClientConnectionId\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\x12\x13\n\x0bsrv_version\x18\x03 \x01(\t\x12\x14\n\x0crate_limiter\x18\x04 \x01(\x05\x12\x10\n\x08\x65xchange\x18\x05 \x01(\t\"=\n\x16\x46\x65tchServerTimeRequest\x12\x11\n\tclient_id\x18\x01 \x01(\x03\x12\x10\n\x08trade_id\x18\x02 \x01(\t\".\n\x17\x46\x65tchServerTimeResponse\x12\x13\n\x0bserver_time\x18\x01 \x01(\x04\x32\xf8\x0e\n\x06Martin\x12]\n\x14OpenClientConnection\x12#.martin.OpenClientConnectionRequest\x1a\x1e.martin.OpenClientConnectionId\"\x00\x12T\n\x0f\x46\x65tchServerTime\x12\x1e.martin.OpenClientConnectionId\x1a\x1f.martin.FetchServerTimeResponse\"\x00\x12K\n\x0f\x46\x65tchOpenOrders\x12\x15.martin.MarketRequest\x1a\x1f.martin.FetchOpenOrdersResponse\"\x00\x12K\n\x0f\x43\x61ncelAllOrders\x12\x15.martin.MarketRequest\x1a\x1f.martin.FetchOpenOrdersResponse\"\x00\x12[\n\x17\x46\x65tchExchangeInfoSymbol\x12\x15.martin.MarketRequest\x1a\'.martin.FetchExchangeInfoSymbolResponse\"\x00\x12`\n\x17\x46\x65tchAccountInformation\x12\x1e.martin.OpenClientConnectionId\x1a#.martin.FetchAccountBalanceResponse\"\x00\x12I\n\x0e\x46\x65tchOrderBook\x12\x15.martin.MarketRequest\x1a\x1e.martin.FetchOrderBookResponse\"\x00\x12Y\n\x16\x46\x65tchSymbolPriceTicker\x12\x15.martin.MarketRequest\x1a&.martin.FetchSymbolPriceTickerResponse\"\x00\x12m\n FetchTickerPriceChangeStatistics\x12\x15.martin.MarketRequest\x1a\x30.martin.FetchTickerPriceChangeStatisticsResponse\"\x00\x12H\n\x0b\x46\x65tchKlines\x12\x1a.martin.FetchKlinesRequest\x1a\x1b.martin.FetchKlinesResponse\"\x00\x12\\\n\x15\x46\x65tchAccountTradeList\x12\x1f.martin.AccountTradeListRequest\x1a .martin.AccountTradeListResponse\"\x00\x12K\n\x0eOnTickerUpdate\x12\x15.martin.MarketRequest\x1a\x1e.martin.OnTickerUpdateResponse\"\x00\x30\x01\x12N\n\x11OnOrderBookUpdate\x12\x15.martin.MarketRequest\x1a\x1e.martin.FetchOrderBookResponse\"\x00\x30\x01\x12=\n\nStopStream\x12\x15.martin.MarketRequest\x1a\x16.martin.SimpleResponse\"\x00\x12\x43\n\x0bStartStream\x12\x1a.martin.StartStreamRequest\x1a\x16.martin.SimpleResponse\"\x00\x12P\n\rOnFundsUpdate\x12\x1c.martin.OnFundsUpdateRequest\x1a\x1d.martin.OnFundsUpdateResponse\"\x00\x30\x01\x12I\n\rOnOrderUpdate\x12\x15.martin.MarketRequest\x1a\x1d.martin.OnOrderUpdateResponse\"\x00\x30\x01\x12W\n\x10\x43reateLimitOrder\x12\x1f.martin.CreateLimitOrderRequest\x1a .martin.CreateLimitOrderResponse\"\x00\x12H\n\x0b\x43\x61ncelOrder\x12\x1a.martin.CancelOrderRequest\x1a\x1b.martin.CancelOrderResponse\"\x00\x12\x45\n\nFetchOrder\x12\x19.martin.FetchOrderRequest\x1a\x1a.martin.FetchOrderResponse\"\x00\x12J\n\x0eResetRateLimit\x12\x1e.martin.OpenClientConnectionId\x1a\x16.martin.SimpleResponse\"\x00\x12P\n\x0eOnKlinesUpdate\x12\x1a.martin.FetchKlinesRequest\x1a\x1e.martin.OnKlinesUpdateResponse\"\x00\x30\x01\x12]\n\x12\x46\x65tchFundingWallet\x12!.martin.FetchFundingWalletRequest\x1a\".martin.FetchFundingWalletResponse\"\x00\x62\x06proto3')



_FETCHFUNDINGWALLETREQUEST = DESCRIPTOR.message_types_by_name['FetchFundingWalletRequest']
_FETCHFUNDINGWALLETRESPONSE = DESCRIPTOR.message_types_by_name['FetchFundingWalletResponse']
_FETCHFUNDINGWALLETRESPONSE_BALANCES = _FETCHFUNDINGWALLETRESPONSE.nested_types_by_name['Balances']
_CANCELORDERRESPONSE = DESCRIPTOR.message_types_by_name['CancelOrderResponse']
_CANCELORDERREQUEST = DESCRIPTOR.message_types_by_name['CancelOrderRequest']
_CREATELIMITORDERRESPONSE = DESCRIPTOR.message_types_by_name['CreateLimitOrderResponse']
_CREATELIMITORDERREQUEST = DESCRIPTOR.message_types_by_name['CreateLimitOrderRequest']
_ONORDERUPDATERESPONSE = DESCRIPTOR.message_types_by_name['OnOrderUpdateResponse']
_ONFUNDSUPDATERESPONSE = DESCRIPTOR.message_types_by_name['OnFundsUpdateResponse']
_ONFUNDSUPDATEREQUEST = DESCRIPTOR.message_types_by_name['OnFundsUpdateRequest']
_SIMPLERESPONSE = DESCRIPTOR.message_types_by_name['SimpleResponse']
_ONTICKERUPDATERESPONSE = DESCRIPTOR.message_types_by_name['OnTickerUpdateResponse']
_ACCOUNTTRADELISTRESPONSE = DESCRIPTOR.message_types_by_name['AccountTradeListResponse']
_ACCOUNTTRADELISTRESPONSE_TRADE = _ACCOUNTTRADELISTRESPONSE.nested_types_by_name['Trade']
_ACCOUNTTRADELISTREQUEST = DESCRIPTOR.message_types_by_name['AccountTradeListRequest']
_FETCHKLINESRESPONSE = DESCRIPTOR.message_types_by_name['FetchKlinesResponse']
_ONKLINESUPDATERESPONSE = DESCRIPTOR.message_types_by_name['OnKlinesUpdateResponse']
_FETCHKLINESREQUEST = DESCRIPTOR.message_types_by_name['FetchKlinesRequest']
_FETCHTICKERPRICECHANGESTATISTICSRESPONSE = DESCRIPTOR.message_types_by_name['FetchTickerPriceChangeStatisticsResponse']
_FETCHSYMBOLPRICETICKERRESPONSE = DESCRIPTOR.message_types_by_name['FetchSymbolPriceTickerResponse']
_FETCHORDERBOOKRESPONSE = DESCRIPTOR.message_types_by_name['FetchOrderBookResponse']
_FETCHACCOUNTBALANCERESPONSE = DESCRIPTOR.message_types_by_name['FetchAccountBalanceResponse']
_FETCHACCOUNTBALANCERESPONSE_BALANCES = _FETCHACCOUNTBALANCERESPONSE.nested_types_by_name['Balances']
_FETCHEXCHANGEINFOSYMBOLRESPONSE = DESCRIPTOR.message_types_by_name['FetchExchangeInfoSymbolResponse']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS = _FETCHEXCHANGEINFOSYMBOLRESPONSE.nested_types_by_name['Filters']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PRICEFILTER = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['PriceFilter']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PERCENTPRICE = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['PercentPrice']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_LOTSIZE = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['LotSize']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MINNOTIONAL = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MinNotional']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_ICEBERGPARTS = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['IcebergParts']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MARKETLOTSIZE = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MarketLotSize']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMORDERS = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MaxNumOrders']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMALGOORDERS = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MaxNumAlgoOrders']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMICEBERGORDERS = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MaxNumIcebergOrders']
_FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXPOSITION = _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS.nested_types_by_name['MaxPosition']
_CANCELALLORDERSRESPONSE = DESCRIPTOR.message_types_by_name['CancelAllOrdersResponse']
_CANCELALLORDERSRESPONSE_CANCELORDER = _CANCELALLORDERSRESPONSE.nested_types_by_name['CancelOrder']
_FETCHORDERREQUEST = DESCRIPTOR.message_types_by_name['FetchOrderRequest']
_FETCHORDERRESPONSE = DESCRIPTOR.message_types_by_name['FetchOrderResponse']
_FETCHOPENORDERSRESPONSE = DESCRIPTOR.message_types_by_name['FetchOpenOrdersResponse']
_FETCHOPENORDERSRESPONSE_ORDER = _FETCHOPENORDERSRESPONSE.nested_types_by_name['Order']
_MARKETREQUEST = DESCRIPTOR.message_types_by_name['MarketRequest']
_STARTSTREAMREQUEST = DESCRIPTOR.message_types_by_name['StartStreamRequest']
_OPENCLIENTCONNECTIONREQUEST = DESCRIPTOR.message_types_by_name['OpenClientConnectionRequest']
_OPENCLIENTCONNECTIONID = DESCRIPTOR.message_types_by_name['OpenClientConnectionId']
_FETCHSERVERTIMEREQUEST = DESCRIPTOR.message_types_by_name['FetchServerTimeRequest']
_FETCHSERVERTIMERESPONSE = DESCRIPTOR.message_types_by_name['FetchServerTimeResponse']
FetchFundingWalletRequest = _reflection.GeneratedProtocolMessageType('FetchFundingWalletRequest', (_message.Message,), {
  'DESCRIPTOR' : _FETCHFUNDINGWALLETREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchFundingWalletRequest)
  })
_sym_db.RegisterMessage(FetchFundingWalletRequest)

FetchFundingWalletResponse = _reflection.GeneratedProtocolMessageType('FetchFundingWalletResponse', (_message.Message,), {

  'Balances' : _reflection.GeneratedProtocolMessageType('Balances', (_message.Message,), {
    'DESCRIPTOR' : _FETCHFUNDINGWALLETRESPONSE_BALANCES,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.FetchFundingWalletResponse.Balances)
    })
  ,
  'DESCRIPTOR' : _FETCHFUNDINGWALLETRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchFundingWalletResponse)
  })
_sym_db.RegisterMessage(FetchFundingWalletResponse)
_sym_db.RegisterMessage(FetchFundingWalletResponse.Balances)

CancelOrderResponse = _reflection.GeneratedProtocolMessageType('CancelOrderResponse', (_message.Message,), {
  'DESCRIPTOR' : _CANCELORDERRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.CancelOrderResponse)
  })
_sym_db.RegisterMessage(CancelOrderResponse)

CancelOrderRequest = _reflection.GeneratedProtocolMessageType('CancelOrderRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELORDERREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.CancelOrderRequest)
  })
_sym_db.RegisterMessage(CancelOrderRequest)

CreateLimitOrderResponse = _reflection.GeneratedProtocolMessageType('CreateLimitOrderResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATELIMITORDERRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.CreateLimitOrderResponse)
  })
_sym_db.RegisterMessage(CreateLimitOrderResponse)

CreateLimitOrderRequest = _reflection.GeneratedProtocolMessageType('CreateLimitOrderRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATELIMITORDERREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.CreateLimitOrderRequest)
  })
_sym_db.RegisterMessage(CreateLimitOrderRequest)

OnOrderUpdateResponse = _reflection.GeneratedProtocolMessageType('OnOrderUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _ONORDERUPDATERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OnOrderUpdateResponse)
  })
_sym_db.RegisterMessage(OnOrderUpdateResponse)

OnFundsUpdateResponse = _reflection.GeneratedProtocolMessageType('OnFundsUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _ONFUNDSUPDATERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OnFundsUpdateResponse)
  })
_sym_db.RegisterMessage(OnFundsUpdateResponse)

OnFundsUpdateRequest = _reflection.GeneratedProtocolMessageType('OnFundsUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _ONFUNDSUPDATEREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OnFundsUpdateRequest)
  })
_sym_db.RegisterMessage(OnFundsUpdateRequest)

SimpleResponse = _reflection.GeneratedProtocolMessageType('SimpleResponse', (_message.Message,), {
  'DESCRIPTOR' : _SIMPLERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.SimpleResponse)
  })
_sym_db.RegisterMessage(SimpleResponse)

OnTickerUpdateResponse = _reflection.GeneratedProtocolMessageType('OnTickerUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _ONTICKERUPDATERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OnTickerUpdateResponse)
  })
_sym_db.RegisterMessage(OnTickerUpdateResponse)

AccountTradeListResponse = _reflection.GeneratedProtocolMessageType('AccountTradeListResponse', (_message.Message,), {

  'Trade' : _reflection.GeneratedProtocolMessageType('Trade', (_message.Message,), {
    'DESCRIPTOR' : _ACCOUNTTRADELISTRESPONSE_TRADE,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.AccountTradeListResponse.Trade)
    })
  ,
  'DESCRIPTOR' : _ACCOUNTTRADELISTRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.AccountTradeListResponse)
  })
_sym_db.RegisterMessage(AccountTradeListResponse)
_sym_db.RegisterMessage(AccountTradeListResponse.Trade)

AccountTradeListRequest = _reflection.GeneratedProtocolMessageType('AccountTradeListRequest', (_message.Message,), {
  'DESCRIPTOR' : _ACCOUNTTRADELISTREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.AccountTradeListRequest)
  })
_sym_db.RegisterMessage(AccountTradeListRequest)

FetchKlinesResponse = _reflection.GeneratedProtocolMessageType('FetchKlinesResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHKLINESRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchKlinesResponse)
  })
_sym_db.RegisterMessage(FetchKlinesResponse)

OnKlinesUpdateResponse = _reflection.GeneratedProtocolMessageType('OnKlinesUpdateResponse', (_message.Message,), {
  'DESCRIPTOR' : _ONKLINESUPDATERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OnKlinesUpdateResponse)
  })
_sym_db.RegisterMessage(OnKlinesUpdateResponse)

FetchKlinesRequest = _reflection.GeneratedProtocolMessageType('FetchKlinesRequest', (_message.Message,), {
  'DESCRIPTOR' : _FETCHKLINESREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchKlinesRequest)
  })
_sym_db.RegisterMessage(FetchKlinesRequest)

FetchTickerPriceChangeStatisticsResponse = _reflection.GeneratedProtocolMessageType('FetchTickerPriceChangeStatisticsResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHTICKERPRICECHANGESTATISTICSRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchTickerPriceChangeStatisticsResponse)
  })
_sym_db.RegisterMessage(FetchTickerPriceChangeStatisticsResponse)

FetchSymbolPriceTickerResponse = _reflection.GeneratedProtocolMessageType('FetchSymbolPriceTickerResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHSYMBOLPRICETICKERRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchSymbolPriceTickerResponse)
  })
_sym_db.RegisterMessage(FetchSymbolPriceTickerResponse)

FetchOrderBookResponse = _reflection.GeneratedProtocolMessageType('FetchOrderBookResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHORDERBOOKRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchOrderBookResponse)
  })
_sym_db.RegisterMessage(FetchOrderBookResponse)

FetchAccountBalanceResponse = _reflection.GeneratedProtocolMessageType('FetchAccountBalanceResponse', (_message.Message,), {

  'Balances' : _reflection.GeneratedProtocolMessageType('Balances', (_message.Message,), {
    'DESCRIPTOR' : _FETCHACCOUNTBALANCERESPONSE_BALANCES,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.FetchAccountBalanceResponse.Balances)
    })
  ,
  'DESCRIPTOR' : _FETCHACCOUNTBALANCERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchAccountBalanceResponse)
  })
_sym_db.RegisterMessage(FetchAccountBalanceResponse)
_sym_db.RegisterMessage(FetchAccountBalanceResponse.Balances)

FetchExchangeInfoSymbolResponse = _reflection.GeneratedProtocolMessageType('FetchExchangeInfoSymbolResponse', (_message.Message,), {

  'Filters' : _reflection.GeneratedProtocolMessageType('Filters', (_message.Message,), {

    'PriceFilter' : _reflection.GeneratedProtocolMessageType('PriceFilter', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PRICEFILTER,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.PriceFilter)
      })
    ,

    'PercentPrice' : _reflection.GeneratedProtocolMessageType('PercentPrice', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PERCENTPRICE,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.PercentPrice)
      })
    ,

    'LotSize' : _reflection.GeneratedProtocolMessageType('LotSize', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_LOTSIZE,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.LotSize)
      })
    ,

    'MinNotional' : _reflection.GeneratedProtocolMessageType('MinNotional', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MINNOTIONAL,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MinNotional)
      })
    ,

    'IcebergParts' : _reflection.GeneratedProtocolMessageType('IcebergParts', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_ICEBERGPARTS,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.IcebergParts)
      })
    ,

    'MarketLotSize' : _reflection.GeneratedProtocolMessageType('MarketLotSize', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MARKETLOTSIZE,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MarketLotSize)
      })
    ,

    'MaxNumOrders' : _reflection.GeneratedProtocolMessageType('MaxNumOrders', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMORDERS,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumOrders)
      })
    ,

    'MaxNumAlgoOrders' : _reflection.GeneratedProtocolMessageType('MaxNumAlgoOrders', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMALGOORDERS,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumAlgoOrders)
      })
    ,

    'MaxNumIcebergOrders' : _reflection.GeneratedProtocolMessageType('MaxNumIcebergOrders', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMICEBERGORDERS,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MaxNumIcebergOrders)
      })
    ,

    'MaxPosition' : _reflection.GeneratedProtocolMessageType('MaxPosition', (_message.Message,), {
      'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXPOSITION,
      '__module__' : 'exchanges_wrapper.api_pb2'
      # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters.MaxPosition)
      })
    ,
    'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse.Filters)
    })
  ,
  'DESCRIPTOR' : _FETCHEXCHANGEINFOSYMBOLRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchExchangeInfoSymbolResponse)
  })
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.PriceFilter)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.PercentPrice)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.LotSize)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MinNotional)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.IcebergParts)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MarketLotSize)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MaxNumOrders)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MaxNumAlgoOrders)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MaxNumIcebergOrders)
_sym_db.RegisterMessage(FetchExchangeInfoSymbolResponse.Filters.MaxPosition)

CancelAllOrdersResponse = _reflection.GeneratedProtocolMessageType('CancelAllOrdersResponse', (_message.Message,), {

  'CancelOrder' : _reflection.GeneratedProtocolMessageType('CancelOrder', (_message.Message,), {
    'DESCRIPTOR' : _CANCELALLORDERSRESPONSE_CANCELORDER,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.CancelAllOrdersResponse.CancelOrder)
    })
  ,
  'DESCRIPTOR' : _CANCELALLORDERSRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.CancelAllOrdersResponse)
  })
_sym_db.RegisterMessage(CancelAllOrdersResponse)
_sym_db.RegisterMessage(CancelAllOrdersResponse.CancelOrder)

FetchOrderRequest = _reflection.GeneratedProtocolMessageType('FetchOrderRequest', (_message.Message,), {
  'DESCRIPTOR' : _FETCHORDERREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchOrderRequest)
  })
_sym_db.RegisterMessage(FetchOrderRequest)

FetchOrderResponse = _reflection.GeneratedProtocolMessageType('FetchOrderResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHORDERRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchOrderResponse)
  })
_sym_db.RegisterMessage(FetchOrderResponse)

FetchOpenOrdersResponse = _reflection.GeneratedProtocolMessageType('FetchOpenOrdersResponse', (_message.Message,), {

  'Order' : _reflection.GeneratedProtocolMessageType('Order', (_message.Message,), {
    'DESCRIPTOR' : _FETCHOPENORDERSRESPONSE_ORDER,
    '__module__' : 'exchanges_wrapper.api_pb2'
    # @@protoc_insertion_point(class_scope:martin.FetchOpenOrdersResponse.Order)
    })
  ,
  'DESCRIPTOR' : _FETCHOPENORDERSRESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchOpenOrdersResponse)
  })
_sym_db.RegisterMessage(FetchOpenOrdersResponse)
_sym_db.RegisterMessage(FetchOpenOrdersResponse.Order)

MarketRequest = _reflection.GeneratedProtocolMessageType('MarketRequest', (_message.Message,), {
  'DESCRIPTOR' : _MARKETREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.MarketRequest)
  })
_sym_db.RegisterMessage(MarketRequest)

StartStreamRequest = _reflection.GeneratedProtocolMessageType('StartStreamRequest', (_message.Message,), {
  'DESCRIPTOR' : _STARTSTREAMREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.StartStreamRequest)
  })
_sym_db.RegisterMessage(StartStreamRequest)

OpenClientConnectionRequest = _reflection.GeneratedProtocolMessageType('OpenClientConnectionRequest', (_message.Message,), {
  'DESCRIPTOR' : _OPENCLIENTCONNECTIONREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OpenClientConnectionRequest)
  })
_sym_db.RegisterMessage(OpenClientConnectionRequest)

OpenClientConnectionId = _reflection.GeneratedProtocolMessageType('OpenClientConnectionId', (_message.Message,), {
  'DESCRIPTOR' : _OPENCLIENTCONNECTIONID,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.OpenClientConnectionId)
  })
_sym_db.RegisterMessage(OpenClientConnectionId)

FetchServerTimeRequest = _reflection.GeneratedProtocolMessageType('FetchServerTimeRequest', (_message.Message,), {
  'DESCRIPTOR' : _FETCHSERVERTIMEREQUEST,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchServerTimeRequest)
  })
_sym_db.RegisterMessage(FetchServerTimeRequest)

FetchServerTimeResponse = _reflection.GeneratedProtocolMessageType('FetchServerTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHSERVERTIMERESPONSE,
  '__module__' : 'exchanges_wrapper.api_pb2'
  # @@protoc_insertion_point(class_scope:martin.FetchServerTimeResponse)
  })
_sym_db.RegisterMessage(FetchServerTimeResponse)

_MARTIN = DESCRIPTOR.services_by_name['Martin']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _FETCHFUNDINGWALLETREQUEST._serialized_start=40
  _FETCHFUNDINGWALLETREQUEST._serialized_end=171
  _FETCHFUNDINGWALLETRESPONSE._serialized_start=174
  _FETCHFUNDINGWALLETRESPONSE._serialized_end=382
  _FETCHFUNDINGWALLETRESPONSE_BALANCES._serialized_start=267
  _FETCHFUNDINGWALLETRESPONSE_BALANCES._serialized_end=382
  _CANCELORDERRESPONSE._serialized_start=385
  _CANCELORDERRESPONSE._serialized_end=679
  _CANCELORDERREQUEST._serialized_start=681
  _CANCELORDERREQUEST._serialized_end=772
  _CREATELIMITORDERRESPONSE._serialized_start=775
  _CREATELIMITORDERRESPONSE._serialized_end=1047
  _CREATELIMITORDERREQUEST._serialized_start=1050
  _CREATELIMITORDERREQUEST._serialized_end=1208
  _ONORDERUPDATERESPONSE._serialized_start=1211
  _ONORDERUPDATERESPONSE._serialized_end=1971
  _ONFUNDSUPDATERESPONSE._serialized_start=1973
  _ONFUNDSUPDATERESPONSE._serialized_end=2011
  _ONFUNDSUPDATEREQUEST._serialized_start=2013
  _ONFUNDSUPDATEREQUEST._serialized_end=2129
  _SIMPLERESPONSE._serialized_start=2131
  _SIMPLERESPONSE._serialized_end=2164
  _ONTICKERUPDATERESPONSE._serialized_start=2166
  _ONTICKERUPDATERESPONSE._serialized_end=2267
  _ACCOUNTTRADELISTRESPONSE._serialized_start=2270
  _ACCOUNTTRADELISTRESPONSE._serialized_end=2587
  _ACCOUNTTRADELISTRESPONSE_TRADE._serialized_start=2354
  _ACCOUNTTRADELISTRESPONSE_TRADE._serialized_end=2587
  _ACCOUNTTRADELISTREQUEST._serialized_start=2589
  _ACCOUNTTRADELISTREQUEST._serialized_end=2702
  _FETCHKLINESRESPONSE._serialized_start=2704
  _FETCHKLINESRESPONSE._serialized_end=2741
  _ONKLINESUPDATERESPONSE._serialized_start=2743
  _ONKLINESUPDATERESPONSE._serialized_end=2817
  _FETCHKLINESREQUEST._serialized_start=2819
  _FETCHKLINESREQUEST._serialized_end=2925
  _FETCHTICKERPRICECHANGESTATISTICSRESPONSE._serialized_start=2928
  _FETCHTICKERPRICECHANGESTATISTICSRESPONSE._serialized_end=3367
  _FETCHSYMBOLPRICETICKERRESPONSE._serialized_start=3369
  _FETCHSYMBOLPRICETICKERRESPONSE._serialized_end=3432
  _FETCHORDERBOOKRESPONSE._serialized_start=3434
  _FETCHORDERBOOKRESPONSE._serialized_end=3508
  _FETCHACCOUNTBALANCERESPONSE._serialized_start=3511
  _FETCHACCOUNTBALANCERESPONSE._serialized_end=3661
  _FETCHACCOUNTBALANCERESPONSE_BALANCES._serialized_start=267
  _FETCHACCOUNTBALANCERESPONSE_BALANCES._serialized_end=322
  _FETCHEXCHANGEINFOSYMBOLRESPONSE._serialized_start=3664
  _FETCHEXCHANGEINFOSYMBOLRESPONSE._serialized_end=6103
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS._serialized_start=4225
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS._serialized_end=6103
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PRICEFILTER._serialized_start=5150
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PRICEFILTER._serialized_end=5237
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PERCENTPRICE._serialized_start=5239
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_PERCENTPRICE._serialized_end=5341
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_LOTSIZE._serialized_start=5343
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_LOTSIZE._serialized_end=5422
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MINNOTIONAL._serialized_start=5424
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MINNOTIONAL._serialized_end=5523
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_ICEBERGPARTS._serialized_start=5525
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_ICEBERGPARTS._serialized_end=5574
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MARKETLOTSIZE._serialized_start=5576
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MARKETLOTSIZE._serialized_end=5661
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMORDERS._serialized_start=5663
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMORDERS._serialized_end=5719
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMALGOORDERS._serialized_start=5721
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMALGOORDERS._serialized_end=5785
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMICEBERGORDERS._serialized_start=5787
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXNUMICEBERGORDERS._serialized_end=5857
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXPOSITION._serialized_start=5859
  _FETCHEXCHANGEINFOSYMBOLRESPONSE_FILTERS_MAXPOSITION._serialized_end=5913
  _CANCELALLORDERSRESPONSE._serialized_start=6106
  _CANCELALLORDERSRESPONSE._serialized_end=6480
  _CANCELALLORDERSRESPONSE_CANCELORDER._serialized_start=6194
  _CANCELALLORDERSRESPONSE_CANCELORDER._serialized_end=6480
  _FETCHORDERREQUEST._serialized_start=6482
  _FETCHORDERREQUEST._serialized_end=6600
  _FETCHORDERRESPONSE._serialized_start=6603
  _FETCHORDERRESPONSE._serialized_end=6966
  _FETCHOPENORDERSRESPONSE._serialized_start=6969
  _FETCHOPENORDERSRESPONSE._serialized_end=7423
  _FETCHOPENORDERSRESPONSE_ORDER._serialized_start=7073
  _FETCHOPENORDERSRESPONSE_ORDER._serialized_end=7423
  _MARKETREQUEST._serialized_start=7425
  _MARKETREQUEST._serialized_end=7493
  _STARTSTREAMREQUEST._serialized_start=7496
  _STARTSTREAMREQUEST._serialized_end=7625
  _OPENCLIENTCONNECTIONREQUEST._serialized_start=7627
  _OPENCLIENTCONNECTIONREQUEST._serialized_end=7718
  _OPENCLIENTCONNECTIONID._serialized_start=7720
  _OPENCLIENTCONNECTIONID._serialized_end=7842
  _FETCHSERVERTIMEREQUEST._serialized_start=7844
  _FETCHSERVERTIMEREQUEST._serialized_end=7905
  _FETCHSERVERTIMERESPONSE._serialized_start=7907
  _FETCHSERVERTIMERESPONSE._serialized_end=7953
  _MARTIN._serialized_start=7956
  _MARTIN._serialized_end=9868
# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import json
from urllib.parse import urlencode
from exchanges_wrapper import __version__
import logging
import time
from exchanges_wrapper.c_structures import generate_signature
from exchanges_wrapper.errors import (
    RateLimitReached,
    ExchangeError,
    WAFLimitViolated,
    IPAddressBanned,
    HTTPError,
    QueryCanceled,
)

logger = logging.getLogger('exch_srv_logger')


class HttpClient:
    def __init__(self,
                 api_key,
                 api_secret,
                 endpoint,
                 user_agent,
                 proxy,
                 session,
                 exchange,
                 sub_account):
        self.api_key = api_key
        self.api_secret = api_secret
        self.endpoint = endpoint
        self.rate_limit_reached = False
        if user_agent:
            self.user_agent = user_agent
        else:
            self.user_agent = f"exchange-wrapper, {__version__}"
        self.proxy = proxy
        self.session = session
        self.exchange = exchange
        self.sub_account = sub_account

    async def handle_errors(self, response):
        if response.status >= 500:
            raise ExchangeError(f"An issue occurred on exchange's side: {response.status}: {response.url}:"
                                f" {response.reason}")
        if response.status == 429:
            self.rate_limit_reached = True if self.exchange == 'binance' else None
            raise RateLimitReached(RateLimitReached.message)
        # print(f"handle_errors.response: {response}")
        payload = await response.json()
        if payload and "code" in payload:
            # as defined here: https://github.com/binance/binance-spot-api-docs/blob/
            # master/errors.md#error-codes-for-binance-2019-09-25
            raise ExchangeError(payload["msg"])
        if response.status >= 400:
            logger.debug(f"handle_errors.response.status >= 400: {payload}")
            if response.status == 400 and payload and payload.get("error", str()) == "ERR_RATE_LIMIT":
                raise RateLimitReached(RateLimitReached.message)
            elif response.status == 403:
                raise WAFLimitViolated(WAFLimitViolated.message)
            elif response.status == 418:
                raise IPAddressBanned(IPAddressBanned.message)
            else:
                raise HTTPError(f"Malformed request: {payload}")
        return payload

    async def send_api_call(
        self, path, method="GET", signed=False, send_api_key=True, endpoint=None, **kwargs
    ):
        if self.rate_limit_reached:
            raise QueryCanceled(
                "Rate limit reached, to avoid an IP ban, this query has been cancelled"
            )
        # return the JSON body of a call to Binance REST API
        _endpoint = endpoint or self.endpoint
        query_kwargs = {}
        content = str()
        ftx_post = self.exchange == 'ftx' and method == 'POST'
        bfx_post = self.exchange == 'bitfinex' and ((method == 'POST' and kwargs) or "params" in kwargs)
        _params = json.dumps(kwargs) if ftx_post or bfx_post else None
        url = f'{_endpoint}{path}' if self.exchange == 'binance' else f'{_endpoint}/{path}'
        ts = int(time.time() * 1000)
        if self.exchange == 'binance':
            query_kwargs = dict({"headers": {"User-Agent": self.user_agent}}, **kwargs,)
            if send_api_key:
                query_kwargs["headers"]["X-MBX-APIKEY"] = self.api_key
        elif self.exchange in ('ftx', 'bitfinex'):
            # https://help.ftx.com/hc/en-us/articles/360052595091-2020-11-20-Ratelimit-Updates
            query_kwargs = {"headers": {"Accept": 'application/json'}}
            if self.exchange == 'ftx':
                query_kwargs["headers"]["FTX-KEY"] = self.api_key
                if self.sub_account:
                    query_kwargs["headers"]["FTX-SUBACCOUNT"] = self.sub_account
            content += urlencode(kwargs, safe='/')
            if content and not ftx_post and not bfx_post:
                url += f'?{content}'
            if bfx_post and "params" in kwargs:
                query_kwargs.update({'data': _params})
        if signed:
            query_kwargs["headers"]["Content-Type"] = 'application/json'
            if self.exchange == 'binance':
                location = "params" if "params" in kwargs else "data"
                query_kwargs[location]["timestamp"] = str(ts)
                if "params" in kwargs:
                    content += urlencode(kwargs["params"])
                if "data" in kwargs:
                    content += urlencode(kwargs["data"])
                query_kwargs[location]["signature"] = generate_signature(self.exchange, self.api_secret, content)
                if self.proxy:
                    query_kwargs["proxy"] = self.proxy
            elif self.exchange == 'ftx':
                if ftx_post:
                    query_kwargs.update({'data': _params})
                    content = f"{_params}"
                signature_payload = f'{ts}{method}/api/{path}'
                if content:
                    if ftx_post:
                        signature_payload += f'{content}'
                    else:
                        signature_payload += f'?{content}'
                query_kwargs["headers"]["FTX-SIGN"] = generate_signature(self.exchange,
                                                                         self.api_secret,
                                                                         signature_payload)
                query_kwargs["headers"]["FTX-TS"] = str(ts)
            elif self.exchange == 'bitfinex':
                if bfx_post:
                    query_kwargs.update({'data': _params})
                if send_api_key:
                    query_kwargs["headers"]["bfx-apikey"] = self.api_key
                signature_payload = f'/api/{path}{ts}'
                if _params:
                    signature_payload += f"{_params}"
                query_kwargs["headers"]["bfx-signature"] = generate_signature(self.exchange,
                                                                              self.api_secret,
                                                                              signature_payload)
                query_kwargs["headers"]["bfx-nonce"] = str(ts)
        async with self.session.request(method, url, **query_kwargs) as response:
            # logger.debug(f"send_api_call.response: url: {response.url}, status: {response.status}")
            return await self.handle_errors(response)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
Parser for convert FTX REST API/WSS response to Binance like result
"""
import time
import datetime
from decimal import Decimal
import logging
import zlib

logger = logging.getLogger('exch_srv_logger')

CH_KEY = {
    'miniTicker': ['last'],
    'depth5': ['bid', 'ask'],
}
TIMESTAMP_PATTERN = "%Y-%m-%dT%H:%M:%S.%f"


class OrderBook:
    def __init__(self, _order_book, symbol) -> None:
        self.symbol = symbol.replace('/', '').lower()
        self.last_update_id = int(_order_book.get('time', 0) * 1000)
        self.asks = {}
        self.bids = {}
        for i in _order_book.get('bids', []):
            self.bids[str(i[0])] = str(i[1])
        for i in _order_book.get('asks', []):
            self.asks[str(i[0])] = str(i[1])

    def sort_order_book(self) -> ():
        bids = list(map(list, self.bids.items()))
        bids.sort(key=lambda x: float(x[0]), reverse=True)
        asks = list(map(list, self.asks.items()))
        asks.sort(key=lambda x: float(x[0]), reverse=False)
        return bids, asks

    def get_book(self) -> dict:
        bids, asks = self.sort_order_book()
        return {
            'stream': f"{self.symbol}@depth5",
            'data': {'lastUpdateId': self.last_update_id,
                     'bids': bids[0:5],
                     'asks': asks[0:5],
                     }
        }

    def checksum(self) -> int:
        raw = str()
        bids, asks = self.sort_order_book()
        bids_max = len(bids)
        asks_max = len(asks)
        for i in range(max(bids_max, asks_max)):
            if i < bids_max:
                raw += f"{':' if i else ''}{bids[i][0]}:{bids[i][1]}"
            if i < asks_max:
                raw += f":{asks[i][0]}:{asks[i][1]}"
        res = zlib.crc32(raw.encode('utf8'))
        return res

    def update_book(self, _update) -> int:
        self.last_update_id = int(_update.get('time', 0) * 1000)
        bids = _update.get('bids', [])
        for bid in bids:
            if bid[1]:
                self.bids[str(bid[0])] = str(bid[1])
            else:
                self.bids.pop(str(bid[0]), None)

        asks = _update.get('asks', [])
        for ask in asks:
            if ask[1]:
                self.asks[str(ask[0])] = str(ask[1])
            else:
                self.asks.pop(str(ask[0]), None)
        return self.checksum()

    def __call__(self):
        return self


def on_funds_update(res: []) -> {}:
    binance_funds = {
        'e': 'outboundAccountPosition',
        'E': int(time.time() * 1000),
        'u': int(time.time() * 1000),
    }
    funds = []
    for i in res:
        balance = {
            'a': i.get('asset'),
            'f': i.get('free'),
            'l': i.get('locked')
        }
        funds.append(balance)
    binance_funds['B'] = funds
    return binance_funds


def interval(_interval: str) -> int:
    resolution = {
        '1m': 60,
        '5m': 300,
        '15m': 900,
        '1h': 3600,
        '4h': 14400,
        '1d': 86400,
    }
    return resolution.get(_interval, 0)


def exchange_info(res: []) -> {}:
    symbols = []
    for market in res:
        market_type = market.get("type")
        volume_usd24h = market.get("volumeUsd24h")
        if volume_usd24h and market_type == 'spot':
            # print(f"fetch_exchange_info.market: {market}")
            _symbol = str.replace(market.get('name'), '/', '')
            # print(f"fetch_exchange_info._symbol: {_symbol}")
            _base_asset = market.get('baseCurrency')
            _base_asset_precision = len(str(market.get('sizeIncrement'))) - 2
            _quote_asset = market.get('quoteCurrency')
            # Filters var
            _tick_size = market.get('priceIncrement')
            _min_qty = market.get('minProvideSize')
            _max_qty = market.get('largeOrderThreshold')
            _step_size = market.get('sizeIncrement')
            _min_notional = market.get('minProvideSize') * market.get('price')

            _price_filter = {
                "filterType": "PRICE_FILTER",
                "minPrice": str(_tick_size),
                "maxPrice": "100000.00000000",
                "tickSize": str(_tick_size)
            }
            _lot_size = {
                "filterType": "LOT_SIZE",
                "minQty": str(_min_qty),
                "maxQty": str(_max_qty),
                "stepSize": str(_step_size)
            }
            _min_notional = {
                "filterType": "MIN_NOTIONAL",
                "minNotional": str(_min_notional),
                "applyToMarket": True,
                "avgPriceMins": 5
            }

            symbol = {
                "symbol": _symbol,
                "status": "TRADING",
                "baseAsset": _base_asset,
                "baseAssetPrecision": _base_asset_precision,
                "quoteAsset": _quote_asset,
                "quotePrecision": _base_asset_precision,
                "quoteAssetPrecision": _base_asset_precision,
                "baseCommissionPrecision": 8,
                "quoteCommissionPrecision": 8,
                "orderTypes": ["LIMIT", "MARKET"],
                "icebergAllowed": False,
                "ocoAllowed": False,
                "quoteOrderQtyMarketAllowed": False,
                "allowTrailingStop": False,
                "cancelReplaceAllowed": False,
                "isSpotTradingAllowed": True,
                "isMarginTradingAllowed": False,
                "filters": [_price_filter, _lot_size, _min_notional],
                "permissions": ["SPOT"],
            }
            symbols.append(symbol)
            # print(f"fetch_exchange_info.symbol: {symbol}")

    _binance_res = {
        "timezone": "UTC",
        "serverTime": int(time.time() * 1000),
        "rateLimits": [],
        "exchangeFilters": [],
        "symbols": symbols,
    }
    return _binance_res


def order(_order: {}, response_type=None) -> {}:
    # print(f"order._order: {_order}")
    symbol = str.replace(_order.get('market'), '/', '')
    order_id = _order.get('id')
    order_list_id = -1
    client_order_id = _order.get('clientId', str()) or str()
    price = str(_order.get('price') or 0.0)
    orig_qty = str(_order.get('size') or 0.0)
    executed_qty = str(_order.get('filledSize') or 0.0)
    avg_fill_price = str(_order.get('avgFillPrice') or 0.0)
    cummulative_quote_qty = str(Decimal(executed_qty) * Decimal(avg_fill_price))
    #
    ftx_status = _order.get('status')
    if ftx_status in ('new', 'open') and not float(executed_qty):
        status = 'NEW'
    elif ftx_status in ('new', 'open') and float(executed_qty) and _order.get('remainingSize'):
        status = 'PARTIALLY_FILLED'
    elif ftx_status == 'closed' and orig_qty == executed_qty:
        status = 'FILLED'
    else:
        status = 'CANCELED'
    #
    time_in_force = "GTC"
    _type = "LIMIT"
    side = _order.get('side').upper()
    stop_price = '0.0'
    iceberg_qty = '0.0'
    #
    ftx_time = _order.get('createdAt')
    _time = int(datetime.datetime.strptime(ftx_time.split('+')[0], TIMESTAMP_PATTERN).timestamp() * 1000)
    #
    update_time = _time
    is_working = True
    orig_quote_order_qty = str(Decimal(orig_qty) * Decimal(price))
    if response_type:
        binance_order = {
            "symbol": symbol,
            "origClientOrderId": client_order_id,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "transactTime": _time,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
        }
    elif response_type is None:
        binance_order = {
            "symbol": symbol,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
            "stopPrice": stop_price,
            "icebergQty": iceberg_qty,
            "time": _time,
            "updateTime": update_time,
            "isWorking": is_working,
            "origQuoteOrderQty": orig_quote_order_qty,
        }
    else:
        binance_order = {
            "symbol": symbol,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
        }
    # print(f"order.binance_order: {binance_order}")
    return binance_order


def orders(res: {}, response_type=None) -> []:
    # print(f"orders.res: {res}")
    binance_orders = []
    for _order in res:
        i_order = order(_order, response_type=response_type)
        binance_orders.append(i_order)
    # print(f"orders.binance_orders: {binance_orders}")
    return binance_orders


def fetch_funding_wallet(res: []) -> []:
    balances = []
    for balance in res:
        free = str(balance.get('free') or 0.0)
        total = str(balance.get('total') or 0.0)
        locked = str(Decimal(total) - Decimal(free))
        if float(total):
            _binance_res = {
                "asset": balance.get('coin'),
                "free": free,
                "locked": locked,
                "freeze": "0",
                "withdrawing": "0",
                "btcValuation": "0.0",
            }
            balances.append(_binance_res)
    return balances


def account_information(res: []) -> {}:
    balances = []
    for balance in res:
        free = str(balance.get('free') or 0.0)
        total = str(balance.get('total') or 0.0)
        locked = str(Decimal(total) - Decimal(free))
        _binance_res = {
            "asset": balance.get('coin'),
            "free": free,
            "locked": locked,
        }
        balances.append(_binance_res)

    binance_account_info = {
      "makerCommission": 0,
      "takerCommission": 0,
      "buyerCommission": 0,
      "sellerCommission": 0,
      "canTrade": True,
      "canWithdraw": False,
      "canDeposit": False,
      "updateTime": int(time.time() * 1000),
      "accountType": "SPOT",
      "balances": balances,
      "permissions": [
        "SPOT"
      ]
    }

    return binance_account_info


def order_book(res: {}) -> {}:
    binance_order_book = {"lastUpdateId": int(time.time() * 1000)}
    binance_order_book.update(
        {key: [[str(y) for y in x] for x in value] for key, value in res.items()}
    )
    return binance_order_book


def symbol_price_ticker(res: [], symbol):
    if symbol:
        binance_price_ticker = {
            "symbol": symbol,
            "price": str(res.get('price') or 0.0),
        }
    else:
        binance_price_ticker = []
        for i in res:
            price_ticker = {
                "symbol": i.get('name').replace('/', ''),
                "price": str(i.get('price') or 0.0),
            }
            binance_price_ticker.append(price_ticker)
    return binance_price_ticker


def ticker_price_change_statistics(res: [], symbol, end_time):
    _res = res[-1]
    ftx_time = _res.get('startTime')
    _time = int(datetime.datetime.strptime(ftx_time.split('+')[0], "%Y-%m-%dT%H:%M:%S").timestamp())
    binance_price_ticker = {
        "symbol": symbol,
        "priceChange": "0.0",
        "priceChangePercent": "0.0",
        "weightedAvgPrice": "0.0",
        "prevClosePrice": str(_res.get('open') or 0.0),
        "lastPrice": str(_res.get('close') or 0.0),
        "lastQty": "0.0",
        "bidPrice": "0.0",
        "bidQty": "0.0",
        "askPrice": "0.0",
        "askQty": "0.00",
        "openPrice": str(_res.get('open') or 0.0),
        "highPrice": str(_res.get('high') or 0.0),
        "lowPrice": str(_res.get('low') or 0.0),
        "volume": str(_res.get('volume') or 0.0),
        "quoteVolume": "0.0",
        "openTime": _time * 1000,
        "closeTime": end_time * 1000,
        "firstId": 0,
        "lastId": 1,
        "count": 1,
    }
    return binance_price_ticker


def klines(res: [], _interval: int) -> []:
    binance_klines = []
    for i in res:
        start_time = int(datetime.datetime.strptime(i.get('startTime').split('+')[0],
                                                    "%Y-%m-%dT%H:%M:%S").timestamp() * 1000)
        candle = [
            start_time,
            str(i.get('open') or 0.0),
            str(i.get('high') or 0.0),
            str(i.get('low') or 0.0),
            str(i.get('close') or 0.0),
            str(i.get('volume') or 0.0),
            start_time + _interval * 1000,
            '0.0',
            0,
            '0.0',
            '0.0',
            '0.0',
        ]
        binance_klines.append(candle)
    return binance_klines


def account_trade_list(res: []) -> []:
    binance_trade_list = []
    for trade in res:
        price = str(trade.get('price') or 0.0)
        qty = str(trade.get('size') or 0.0)
        quote_qty = str(Decimal(price) * Decimal(qty))
        ftx_time = trade.get('time')
        _time = int(datetime.datetime.strptime(ftx_time.split('+')[0], TIMESTAMP_PATTERN).timestamp() * 1000)
        binance_trade = {
            "symbol": trade.get('market').replace('/', ''),
            "id": trade.get('tradeId'),
            "orderId": trade.get('orderId'),
            "orderListId": -1,
            "price": price,
            "qty": qty,
            "quoteQty": quote_qty,
            "commission": str(trade.get('fee')),
            "commissionAsset": trade.get('feeCurrency'),
            "time": _time,
            "isBuyer": bool(trade.get('liquidity') == 'taker'),
            "isMaker": bool(trade.get('liquidity') == 'maker'),
            "isBestMatch": True,
        }
        binance_trade_list.append(binance_trade)
    return binance_trade_list


# noinspection PyUnboundLocalVariable
def stream_convert(msg: {}, symbol: str = None, ch_type: str = None) -> {}:
    # print(f"stream_convert: symbol: {symbol}, ch_type: {ch_type}, msg: {msg}")
    msg_binance = {}
    data = msg.get('data')
    _symbol = (symbol if symbol else data.get('market')).replace('/', '').lower()
    if ch_type == 'ticker':
        msg_binance = {
            'stream': f"{_symbol}@miniTicker",
            'data': {
                "e": "24hrMiniTicker",
                "E": int(data.get('time') * 1000),
                "s": _symbol.upper(),
                "c": str(data.get('last') or 0.0),
                "o": "0.0",
                "h": "0.0",
                "l": "0.0",
                "v": "0",
                "q": "0"
            }
        }
    else:
        ftx_time = None
        if msg.get('channel') == 'orders' and data.get('status') == 'closed':
            ftx_time = data.get('createdAt')
            client_order_id = data.get('clientId')
            order_quantity = str(data.get('size'))
            order_price = str(data.get('price'))
            order_status = 'FILLED' if data.get('filledSize') == data.get('size') else 'CANCELED'
            order_id = data.get('id')
            last_executed_quantity = str(0.0)
            cumulative_filled_quantity = str(data.get('filledSize') or 0.0)
            last_executed_price = str(data.get('avgFillPrice') or 0.0)
            trade_id = -1
            cumulative_quote_asset = str(Decimal(cumulative_filled_quantity) * Decimal(last_executed_price))
            last_quote_asset = cumulative_quote_asset
            quote_order_qty = str(Decimal(order_quantity) * Decimal(order_price))
        elif msg.get('channel') == 'fills':
            ftx_time = data.get('time')
            client_order_id = data.get('clientOrderId')
            order_quantity = '0.0'
            order_price = '0.0'
            order_status = 'PARTIALLY_FILLED'
            order_id = data.get('orderId')
            last_executed_quantity = str(data.get('size') or 0.0)
            cumulative_filled_quantity = '0.0'
            last_executed_price = str(data.get('price') or 0.0)
            trade_id = data.get('tradeId')
            cumulative_quote_asset = '0.0'
            last_quote_asset = str(Decimal(last_executed_quantity) * Decimal(last_executed_price))
            quote_order_qty = '0.0'
        if ftx_time:
            _time = int(datetime.datetime.strptime(ftx_time.split('+')[0], TIMESTAMP_PATTERN).timestamp() * 1000)
            msg_binance = {
                "e": "executionReport",
                "E": _time,
                "s": _symbol.upper(),
                "c": client_order_id or str(),
                "S": data.get('side').upper(),
                "o": "LIMIT",
                "f": "GTC",
                "q": order_quantity,
                "p": order_price,
                "P": "0.00000000",
                "F": "0.00000000",
                "g": -1,
                "C": "",
                "x": "TRADE",
                "X": order_status,
                "r": "NONE",
                "i": order_id,
                "l": last_executed_quantity,
                "z": cumulative_filled_quantity,
                "L": last_executed_price,
                "n": '0.0',
                "N": "NONE",
                "T": _time,
                "t": trade_id,
                "I": 123456789,
                "w": True,
                "m": False,
                "M": False,
                "O": _time,
                "Z": cumulative_quote_asset,
                "Y": last_quote_asset,
                "Q": quote_order_qty
            }
    return msg_binance



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
REST API and WebSocket asyncio wrapper with grpc powered multiplexer server for crypto exchanges
 (Binance, FTX, Bitfinex,)
Utilizes one connection for many accounts and trading pairs.
For SPOT market only
"""
__authors__ = ["Th0rgal", "Jerry Fedorenko"]
__license__ = "MIT"
__maintainer__ = "Jerry Fedorenko"
__contact__ = "https://github.com/DogsTailFarmer"
__email__ = "jerry.fedorenko@yahoo.com"
__credits__ = ["https://github.com/DanyaSWorlD"]
__version__ = "1.2.5-3"

from pathlib import Path
import shutil
#
import platform
print(f"Python {platform.python_version()}")
#
WORK_PATH = Path(Path.home(), ".MartinBinance")
CONFIG_PATH = Path(WORK_PATH, "config")
CONFIG_FILE = Path(CONFIG_PATH, "exch_srv_cfg.toml")
LOG_PATH = Path(WORK_PATH, "exch_srv_log")
LOG_FILE = Path(LOG_PATH, "exch_srv.log")

if CONFIG_FILE.exists():
    print(f"Config found at {CONFIG_FILE}")
else:
    print("Can't find config file! Creating it...")
    CONFIG_PATH.mkdir(parents=True, exist_ok=True)
    LOG_PATH.mkdir(parents=True, exist_ok=True)
    shutil.copy(Path(Path(__file__).parent.absolute(), "exch_srv_cfg.toml.template"), CONFIG_FILE)
    print(f"Before first run place account(s) API key into {CONFIG_FILE}")
    raise SystemExit(1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

# Enum definitions of the exchanges_wrapper REST API
# see: https://github.com/binance/binance-spot-api-docs/blob/master/rest-api.md#enum-definitions

from enum import Enum


# Symbol status (status)
class SymbolStatus(Enum):
    PRE_TRADING = "PRE_TRADING"
    TRADING = "TRADING"
    POST_TRADING = "POST_TRADING"
    END_OF_DAY = "END_OF_DAY"
    HALT = "HALT"
    AUCTION_MATCH = "AUCTION_MATCH"
    BREAK = "BREAK"


# Symbol type:
class SymbolType(Enum):
    SPOT = "SPOT"


# Order status (status):
class OrderStatus(Enum):
    NEW = "NEW"  # The order has been accepted by the engine.
    PARTIALLY_FILLED = "PARTIALLY_FILLED"  # A part of the order has been filled.
    FILLED = "FILLED"  # The order has been completely filled.
    CANCELED = "CANCELED"  # The order has been canceled by the user.
    PENDING_CANCEL = "PENDING_CANCEL"  # (currently unused)
    REJECTED = "REJECTED"  # The order was not accepted by the engine and not processed.
    EXPIRED = "EXPIRED"  # The order was canceled according to the order type's rules
    # (e.g. LIMIT FOK orders with no fill, LIMIT IOC or MARKET orders that partially
    # fill) or by the exchange, (e.g. orders canceled during liquidation, orders
    # canceled during maintenance)


# OCO Status (listStatusType):
class ListStatusType(Enum):
    RESPONSE = "RESPONSE"
    EXEC_STARTED = "EXEC_STARTED"
    ALL_DONE = "ALL_DONE"


# OCO Order Status (listOrderStatus):
class ListOrderStatus(Enum):
    EXECUTING = "EXECUTING"
    ALL_DONE = "ALL_DONE"
    REJECT = "REJECT"


# ContingencyType
class ContingencyType(Enum):
    OCO = "OCO"


# Order types (orderTypes, type)
class OrderType(Enum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"
    STOP_LOSS = "STOP_LOSS"
    STOP_LOSS_LIMIT = "STOP_LOSS_LIMIT"
    TAKE_PROFIT = "TAKE_PROFIT"
    TAKE_PROFIT_LIMIT = "TAKE_PROFIT_LIMIT"
    LIMIT_MAKER = "LIMIT_MAKER"


class ResponseType(Enum):
    ACK = "ACK"
    RESULT = "RESULT"
    FULL = "FULL"


# Order side (side)
class Side(Enum):
    BUY = "BUY"
    SELL = "SELL"


# Time in force (timeInForce)
class TimeInForce(Enum):
    GTC = "GTC"
    IOC = "IOC"
    FOK = "FOK"


# Kline/Candlestick chart intervals
# m -> minutes; h -> hours; d -> days; w -> weeks; M -> months
class Interval(Enum):
    ONE_MINUTE = "1m"
    THREE_MINUTES = "3m"
    FIVE_MINUTES = "5m"
    FIFTY_MINUTES = "15m"
    THIRTY_MINUTES = "30m"
    ONE_HOUR = "1h"
    TWO_HOURS = "2h"
    FOUR_HOURS = "4h"
    SIX_HOURS = "6h"
    HEIGHT_HOURS = "8h"
    TWELVE_HOURS = "12h"
    ONE_DAY = "1d"
    THREE_DAY = "3d"
    ONE_WEEK = "1w"
    ONE_MONTH = "1M"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
Parser for convert Bitfinex REST API/WSS response to Binance like result
"""
import time
from decimal import Decimal
import logging

logger = logging.getLogger('exch_srv_logger')


class OrderBook:
    def __init__(self, _order_book, symbol) -> None:
        self.symbol = symbol[1:].replace(':', '').lower()
        self.last_update_id = 1
        self.asks = {}
        self.bids = {}
        for i in _order_book:
            if i[2] > 0:
                self.bids[str(i[0])] = str(i[2])
            else:
                self.asks[str(i[0])] = str(abs(i[2]))

    def get_book(self) -> dict:
        bids = list(map(list, self.bids.items()))
        bids.sort(key=lambda x: float(x[0]), reverse=True)
        asks = list(map(list, self.asks.items()))
        asks.sort(key=lambda x: float(x[0]), reverse=False)
        return {
            'stream': f"{self.symbol}@depth5",
            'data': {'lastUpdateId': self.last_update_id,
                     'bids': bids[0:5],
                     'asks': asks[0:5],
                     }
        }

    def update_book(self, _update):
        self.last_update_id += 1
        if _update[1]:
            if _update[2] > 0:
                self.bids[str(_update[0])] = str(_update[2])
            else:
                self.asks[str(_update[0])] = str(abs(_update[2]))
        else:
            if _update[2] > 0:
                self.bids.pop(str(_update[0]), None)
            else:
                self.asks.pop(str(_update[0]), None)

    def __call__(self):
        return self


def get_symbols(symbols_details: []) -> str:
    symbols = []
    res = ",t"
    for symbol_details in symbols_details:
        symbol = symbol_details['pair']
        if 'f0' not in symbol:
            symbols.append(symbol.upper())
    return f"t{res.join(symbols)}"


def tick_size(precision, _price):
    x = int(_price)
    _price = str(_price)
    if '.' not in _price:
        _price += ".0"
    k = len(_price.split('.')[1])
    x = len(_price.split('.')[0]) if k and x else 0
    if k + x - precision > 0:
        k = precision - x
    elif k + x - precision < 0:
        k += precision - x - k
    res = (1 / 10 ** k) if k else 1
    return res


def symbol_name(_pair: str) -> ():
    if ':' in _pair:
        pair = _pair.replace(':', '').upper()
        base_asset = _pair.split(':')[0].upper()
        quote_asset = _pair.split(':')[1].upper()
    else:
        pair = _pair.upper()
        base_asset = _pair[0:3].upper()
        quote_asset = _pair[3:].upper()
    return pair, base_asset, quote_asset


def exchange_info(symbols_details: [], tickers: []) -> {}:
    symbols = []
    symbols_price = {}
    for pair in tickers:
        symbols_price[pair[0].replace(':', '').upper()[1:]] = pair[7]
    for market in symbols_details:
        if 'f0' not in market.get("pair"):
            _symbol, _base_asset, _quote_asset = symbol_name(market.get("pair"))
            _base_asset_precision = len(str(market.get('minimum_order_size'))) - 2
            # Filters var
            _price = symbols_price.get(_symbol, 0.0)
            _tick_size = tick_size(market.get('price_precision'), _price)
            _min_qty = float(market.get('minimum_order_size'))
            _max_qty = float(market.get('maximum_order_size'))
            _step_size = 0.00001
            _min_notional = _min_qty * _price

            _price_filter = {
                "filterType": "PRICE_FILTER",
                "minPrice": str(_tick_size),
                "maxPrice": "100000.00000000",
                "tickSize": str(_tick_size)
            }
            _lot_size = {
                "filterType": "LOT_SIZE",
                "minQty": str(_min_qty),
                "maxQty": str(_max_qty),
                "stepSize": str(_step_size)
            }
            _min_notional = {
                "filterType": "MIN_NOTIONAL",
                "minNotional": str(_min_notional),
                "applyToMarket": True,
                "avgPriceMins": 5
            }

            symbol = {
                "symbol": _symbol,
                "status": "TRADING",
                "baseAsset": _base_asset,
                "baseAssetPrecision": _base_asset_precision,
                "quoteAsset": _quote_asset,
                "quotePrecision": _base_asset_precision,
                "quoteAssetPrecision": _base_asset_precision,
                "baseCommissionPrecision": 8,
                "quoteCommissionPrecision": 8,
                "orderTypes": ["LIMIT", "MARKET"],
                "icebergAllowed": False,
                "ocoAllowed": False,
                "quoteOrderQtyMarketAllowed": False,
                "allowTrailingStop": False,
                "cancelReplaceAllowed": False,
                "isSpotTradingAllowed": True,
                "isMarginTradingAllowed": False,
                "filters": [_price_filter, _lot_size, _min_notional],
                "permissions": ["SPOT"],
            }
            symbols.append(symbol)

    _binance_res = {
        "timezone": "UTC",
        "serverTime": int(time.time() * 1000),
        "rateLimits": [],
        "exchangeFilters": [],
        "symbols": symbols,
    }
    return _binance_res


def account_information(res: []) -> {}:
    balances = []
    for balance in res:
        if balance[0] == 'exchange':
            total = str(balance[2] or 0.0)
            free = str(balance[4] or 0.0)
            locked = str(Decimal(total) - Decimal(free))
            _binance_res = {
                "asset": balance[1],
                "free": free,
                "locked": locked,
            }
            balances.append(_binance_res)

    binance_account_info = {
      "makerCommission": 0,
      "takerCommission": 0,
      "buyerCommission": 0,
      "sellerCommission": 0,
      "canTrade": True,
      "canWithdraw": False,
      "canDeposit": False,
      "updateTime": int(time.time() * 1000),
      "accountType": "SPOT",
      "balances": balances,
      "permissions": [
        "SPOT"
      ]
    }

    return binance_account_info


def order(res: [], response_type=None, wss_te=None) -> {}:
    # print(f"order.order: {res}")
    symbol = res[3][1:].replace(':', '')
    order_id = res[0]
    order_list_id = -1
    client_order_id = str(res[2]) or str()
    price = str(res[16] or 0.0)
    orig_qty = str(abs(res[7]) or 0.0)
    executed_qty = str(Decimal(orig_qty) - Decimal(str(abs(res[6]))))
    avg_fill_price = str(res[17] or 0.0)
    cummulative_quote_qty = str(Decimal(executed_qty) * Decimal(avg_fill_price))
    orig_quote_order_qty = str(Decimal(orig_qty) * Decimal(price))
    #
    if 'CANCELED' in res[13]:
        status = 'CANCELED'
    elif Decimal(orig_qty) > Decimal(executed_qty) > 0:
        status = 'PARTIALLY_FILLED'
    elif  Decimal(executed_qty) >= Decimal(orig_qty):
        status = 'FILLED'
    else:
        status = 'NEW'
    #
    _type = "LIMIT"
    # https://docs.bitfinex.com/reference/ws-auth-trades
    if wss_te:
        executed_qty = Decimal(str(0))
        cummulative_quote_qty = Decimal(str(0))
        trades_id = []
        for trade in wss_te:
            trade_id = trade[0]
            if trade_id not in trades_id:
                trades_id.append(trade_id)
                exec_amount = Decimal(str(abs(trade[4])))
                exec_price = Decimal(str(trade[5]))
                executed_qty += exec_amount
                cummulative_quote_qty += exec_amount * exec_price
        if executed_qty >= Decimal(orig_qty):
            status = 'FILLED'
        else:
            status = 'PARTIALLY_FILLED'
        executed_qty = str(executed_qty)
        cummulative_quote_qty = str(cummulative_quote_qty)
        _type = "MARKET"

    time_in_force = "GTC"
    side = 'BUY' if res[7] > 0 else 'SELL'
    stop_price = '0.0'
    iceberg_qty = '0.0'
    _time = res[4]
    update_time = res[5]
    is_working = True
    #
    if response_type:
        binance_order = {
            "symbol": symbol,
            "origClientOrderId": client_order_id,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "transactTime": _time,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
        }
    elif response_type is None:
        binance_order = {
            "symbol": symbol,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
            "stopPrice": stop_price,
            "icebergQty": iceberg_qty,
            "time": _time,
            "updateTime": update_time,
            "isWorking": is_working,
            "origQuoteOrderQty": orig_quote_order_qty,
        }
    else:
        binance_order = {
            "symbol": symbol,
            "orderId": order_id,
            "orderListId": order_list_id,
            "clientOrderId": client_order_id,
            "price": price,
            "origQty": orig_qty,
            "executedQty": executed_qty,
            "cummulativeQuoteQty": cummulative_quote_qty,
            "status": status,
            "timeInForce": time_in_force,
            "type": _type,
            "side": side,
        }
    # print(f"order.binance_order: {binance_order}")
    return binance_order


def orders(res: [], response_type=None) -> []:
    binance_orders = []
    for _order in res:
        i_order = order(_order, response_type=response_type)
        binance_orders.append(i_order)
    return binance_orders


def order_book(res: []) -> {}:
    binance_order_book = {"lastUpdateId": int(time.time() * 1000)}
    bids = []
    asks = []
    for i in res:
        if i[2] > 0:
            bids.append([str(i[0]), str(i[2])])
        else:
            asks.append([str(i[0]), str(abs(i[2]))])
    binance_order_book['bids'] = bids
    binance_order_book['asks'] = asks
    return binance_order_book


def ticker_price_change_statistics(res: [], symbol):
    binance_price_ticker = {
        "symbol": symbol,
        "priceChange": str(res[4]),
        "priceChangePercent": str(res[5]),
        "weightedAvgPrice": "0.0",
        "prevClosePrice": str(res[6] - res[4]),
        "lastPrice": str(res[6]),
        "lastQty": "0.0",
        "bidPrice": str(res[0]),
        "bidQty": "0.0",
        "askPrice": str(res[2]),
        "askQty": "0.00",
        "openPrice": str(res[6] - res[4]),
        "highPrice": str(res[8]),
        "lowPrice": str(res[9]),
        "volume": str(res[7]),
        "quoteVolume": "0.0",
        "openTime": int(time.time() * 1000) - 60 * 60 * 24,
        "closeTime": int(time.time() * 1000),
        "firstId": 0,
        "lastId": 1,
        "count": 1,
    }
    return binance_price_ticker


def interval(_interval: str) -> int:
    resolution = {
        '1m': 60,
        '5m': 5 * 60,
        '15m': 15 * 60,
        '30m': 30 * 60,
        '1h': 60 * 60,
        '3h': 3 * 60 * 60,
        '6h': 6 * 60 * 60,
        '12h': 12 * 60 * 60,
        '1D': 24 * 60 * 60,
        '1W': 7 * 24 * 60 * 60,
        '14D': 14 * 24 * 60 * 60,
        '1M': 31 * 24 * 60 * 60
    }
    return resolution.get(_interval, 0)


def klines(res: [], _interval: str) -> []:
    binance_klines = []
    for i in res:
        start_time = i[0]
        _candle = [
            start_time,
            str(i[1]),
            str(i[3]),
            str(i[4]),
            str(i[2]),
            str(i[5]),
            start_time + interval(_interval) * 1000 - 1,
            '0.0',
            0,
            '0.0',
            '0.0',
            '0.0',
        ]
        binance_klines.append(_candle)
    return binance_klines


def candle(res: [], symbol: str = None, ch_type: str = None) -> {}:
    symbol = symbol[1:].replace(':', '')
    start_time = res[0]
    _interval = ch_type.split('_')[1]
    binance_candle = {
        'stream': f"{symbol.lower()}@{ch_type.replace('candles', 'kline')}",
        'data': {'e': 'kline',
                 'E': int(time.time()),
                 's': symbol,
                 'k': {
                     't': start_time,
                     'T': start_time + interval(_interval) * 1000 - 1,
                     's': symbol,
                     'i': _interval,
                     'f': 100,
                     'L': 200,
                     'o': str(res[1]),
                     'c': str(res[2]),
                     'h': str(res[3]),
                     'l': str(res[4]),
                     'v': str(res[5]),
                     'n': 100,
                     'x': False,
                     'q': '0.0',
                     'V': '0.0',
                     'Q': '0.0',
                     'B': '0'}}
    }
    return binance_candle


def account_trade_list(res: []) -> []:
    binance_trade_list = []
    for trade in res:
        price = str(trade[5])
        qty = str(abs(trade[4]))
        quote_qty = str(Decimal(price) * Decimal(qty))
        binance_trade = {
            "symbol": trade[1][1:].replace(':', ''),
            "id": trade[0],
            "orderId": trade[3],
            "orderListId": -1,
            "price": price,
            "qty": qty,
            "quoteQty": quote_qty,
            "commission": str(abs(trade[9])),
            "commissionAsset": trade[10],
            "time": trade[2],
            "isBuyer": bool(trade[4] > 0),
            "isMaker": bool(trade[8] == 1),
            "isBestMatch": True,
        }
        binance_trade_list.append(binance_trade)
    return binance_trade_list


def ticker(res: [], symbol: str = None) -> {}:
    _symbol = symbol[1:].replace(':', '').lower()
    msg_binance = {
        'stream': f"{_symbol}@miniTicker",
        'data': {
            "e": "24hrMiniTicker",
            "E": int(time.time()),
            "s": _symbol.upper(),
            "c": str(res[6]),
            "o": str(res[6] - res[4]),
            "h": str(res[8]),
            "l": str(res[9]),
            "v": str(res[7]),
            "q": "0"
        }
    }
    return msg_binance


def on_funds_update(res: []) -> {}:
    binance_funds = {
        'e': 'outboundAccountPosition',
        'E': int(time.time() * 1000),
        'u': int(time.time() * 1000),
    }
    funds = []
    if not isinstance(res[0], list):
        res = [res]
    for i in res:
        if i[0] == 'exchange':
            total = str(i[2])
            free = str(i[4] or total)
            locked = str(Decimal(total) - Decimal(free))
            balance = {
                'a': i[1],
                'f': free,
                'l': locked
            }
            funds.append(balance)
    binance_funds['B'] = funds
    return binance_funds


def on_order_update(res: [], last_event: tuple) -> {}:
    # print(f"on_order_update.res: {res}")
    side = 'BUY' if res[7] > 0 else 'SELL'
    #
    order_quantity = str(abs(res[7]))
    cumulative_filled_quantity = str(Decimal(order_quantity) - Decimal(str(abs(res[6]))))
    cumulative_quote_asset = str(Decimal(cumulative_filled_quantity) * Decimal(str(res[17])))
    quote_order_qty = str(Decimal(order_quantity) * Decimal(str(res[16])))
    #
    trade_id = -1
    last_executed_quantity = "0"
    last_executed_price = "0"
    if last_event:
        trade_id = last_event[0]
        last_executed_quantity = last_event[1]
        last_executed_price = last_event[2]
    last_quote_asset_transacted = str(Decimal(last_executed_quantity) * Decimal(last_executed_price))
    if 'CANCELED' in res[13]:
        status = 'CANCELED'
    elif Decimal(order_quantity) > Decimal(cumulative_filled_quantity) > 0:
        status = 'PARTIALLY_FILLED'
    elif  Decimal(cumulative_filled_quantity) >= Decimal(order_quantity):
        status = 'FILLED'
    else:
        status = 'NEW'
    #
    msg_binance = {
        "e": "executionReport",
        "E": res[5],
        "s": res[3][1:].replace(':', ''),
        "c": str(res[2]),
        "S": side,
        "o": "LIMIT",
        "f": "GTC",
        "q": order_quantity,
        "p": str(res[16]),
        "P": "0.00000000",
        "F": "0.00000000",
        "g": -1,
        "C": "",
        "x": "TRADE",
        "X": status,
        "r": "NONE",
        "i": res[0],
        "l": last_executed_quantity,
        "z": cumulative_filled_quantity,
        "L": last_executed_price,
        "n": '0.0',
        "N": "NONE",
        "T": res[5],
        "t": trade_id,
        "I": 123456789,
        "w": True,
        "m": False,
        "M": False,
        "O": res[4],
        "Z": cumulative_quote_asset,
        "Y": last_quote_asset_transacted,
        "Q": quote_order_qty
    }
    return msg_binance


def on_order_trade(res: [], executed_qty: str) -> {}:
    # print(f"on_order_trade.res: {res}")
    side = 'BUY' if res[4] > 0 else 'SELL'
    #
    status = 'PARTIALLY_FILLED'
    #
    last_executed_quantity = str(abs(res[4]))
    last_executed_price = str(res[5])
    last_quote_asset = str(Decimal(last_executed_quantity) * Decimal(last_executed_price))
    msg_binance = {
        "e": "executionReport",
        "E": res[2],
        "s": res[1][1:].replace(':', ''),
        "c": str(res[11]),
        "S": side,
        "o": "LIMIT",
        "f": "GTC",
        "q": "0.0",
        "p": str(res[7]),
        "P": "0.00000000",
        "F": "0.00000000",
        "g": -1,
        "C": "NEW",
        "x": "TRADE",
        "X": status,
        "r": "NONE",
        "i": res[3],
        "l": last_executed_quantity,
        "z": executed_qty,
        "L": last_executed_price,
        "n": str(res[9]),
        "N": res[10],
        "T": res[2],
        "t": res[0],
        "I": 123456789,
        "w": True,
        "m": bool(res[8] == 1),
        "M": False,
        "O": res[2],
        "Z": "0.0",
        "Y": last_quote_asset,
        "Q": "0.0"
    }
    return msg_binance


def funding_wallet(res: []) -> []:
    balances = []
    for balance in res:
        if balance[0] in ('exchange', 'funding'):
            total = str(balance[2] or 0.0)
            free = str(balance[4] or 0.0)
            locked = str(Decimal(total) - Decimal(free))
            if float(total):
                _binance_res = {
                    "asset": balance[1],
                    "free": free,
                    "locked": locked,
                    "freeze": "0",
                    "withdrawing": "0",
                    "btcValuation": "0.0",
                }
                balances.append(_binance_res)

    return balances
